import pytest

from mock_call_where.override import AdvancedCall


@pytest.fixture
def mock_where():
    """
    Applies the advanced Call class for the duration of a test, this
    way it is only applied when needed and shouldn't impact other tests.
    """
    AdvancedCall.apply()

    yield

    AdvancedCall.unapply()



import traceback
from unittest import mock


class AdvancedCall(mock._Call):
    """
    An override for the default mock._Call class which adds some additional
    functionality.
    """

    original_call_class = mock._Call

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.where = traceback.format_stack()

    @classmethod
    def apply(cls):
        """
        Apply the override
        """
        mock._Call = cls

    @classmethod
    def unapply(cls):
        """
        Unapply the override
        """
        mock._Call = cls.original_call_class



from unittest import mock

from mock_call_where.override import AdvancedCall


def test_mock_call_has_a_where(mock_where):
    my_mock = mock.Mock()
    my_mock.a_function()
    the_call = my_mock.mock_calls[0]

    assert type(the_call) is AdvancedCall
    assert (
        "in test_mock_call_has_a_where\n    my_mock.a_function()\n"
        in the_call.where[-4]
    )


def test_advanced_call_is_not_automatically_applied():
    my_mock = mock.Mock()

    my_mock.a_function()
    call1 = my_mock.mock_calls[0]
    assert type(call1) is mock._Call

    AdvancedCall.apply()

    my_mock.another_function()
    call2 = my_mock.mock_calls[1]
    assert type(call2) is AdvancedCall

    AdvancedCall.unapply()

    my_mock.a_third_function()
    call3 = my_mock.mock_calls[2]
    assert type(call3) is mock._Call
